﻿-- ----------------------------------------------------------------------- --
--     Group Finder with Chan Buddy                                        
--     Copyright (c) 2006 Moncai                                           
--
--     May be copied and distributed freely as long as you do not
--     change or remove this notice.
--     
-- ----------------------------------------------------------------------- --
CHB_VERSION = GetAddOnMetadata("ChanBuddy", "X-CHBVersion");

-- ------------------------------------------------------------------------------------------------------

function CHB_Supplement_Words()
	for i = 1, GFF.data.num, 1 do
		for idx, key in GFF.data[i].keys do
			CHB_Data.words[key] = 1;
		end
	end
end

function CHB_OnLoad()

	SlashCmdList["CHBCOMMAND"] = CHB_Slash;
	SLASH_CHBCOMMAND1 = "/chb";

	CHB_Init();
	
	this:RegisterEvent("VARIABLES_LOADED");
	
	-- For proper delegates
	this:RegisterEvent("CHAT_MSG_CHANNEL");
	
  	Print(string.format(CHB_LOADED_MSG, CHB_VERSION), 0.0, 1.0, 0.5);
end

-- ------------------------------------------------------------------------------------------------------
local function conf_print(name,var)
	Print(string.format(CHB_CONF_PRINT,name, CHB_TO_E[CHB_Data[var]]), 1.0, 0.8, 0.2);
end
local function conf_toggle(name, var, val)

	if string.find(val,CHB_ON) then 
		CHB_Data[var] = true
	elseif string.find(val,CHB_OFF) then
		CHB_Data[var] = false
	else 
		CHB_Data[var] = not CHB_Data[var];
	end

	conf_print(name,var);
end

function CHB_Slash(msg)
	local firstword, restwords = string.gfind(msg, "([^ ]+) ?(.*)")();
	if restwords == nil then restwords = ""; end
	
	restwords = string.upper(restwords);
	
	if(firstword == "preset" or firstword == "reset") then
		CHB_Data = nil;
		CHB_Init();
		
		if GFF_AnalyseText then
			CHB_Supplement_Words();
			Print("Integrated Group Finder with ChanBuddy", 1.0, 0.5, 0.0);
		end
		
	elseif(firstword == "add") then		
		for word in string.gfind(restwords, "([^ ]+) ?") do
			CHB_Data.words[word] = 1;
		end
	elseif(firstword == "remove") then
		for word in string.gfind(restwords, "([^ ]+) ?") do
			CHB_Data.words[word] = nil;
		end
	elseif(firstword == "filter") then
		conf_toggle("Filter", "bFilter", restwords);
	elseif(firstword == "hide") then
		conf_toggle("Hiding spam", "bHide", restwords);
	elseif(firstword == "highlight") then
		conf_toggle("Highlighting", "bHighlight", restwords);		
	elseif(firstword == "view") then
		local msg, n = ("Current set:"), 1;
		for key,value in CHB_Data.words do
			msg = msg .. " " .. key;
			n = n + 1;
			if n > 10 then
				n = 1;
				Print(""..msg.."", 0.0, 1.0, 0.5);
				msg = "";
			end
		end
		Print(""..msg.."\nType |c00ffcc00/chb help|r for help.", 0.0, 1.0, 0.5);				
	else --if(firstword == "help") then
		for i=1, CHB_HELP_LINE.num, 1 do
			Print(CHB_HELP_LINE[i]);
		end
		conf_print("Filter", "bFilter");
		conf_print("Hiding spam", "bHide");
		conf_print("Highlighting", "bHighlight");
	end
end


function CHB_Init()
	if CHB_Data == nil then
		CHB_Data = {};
		CHB_Data.debug = true;
	end
	CHB_Data.words = CHB_DATA_WORDS;
	
	if CHB_Data.bFilter == nil then CHB_Data.bFilter = true end
	if CHB_Data.bHide == nil then CHB_Data.bHide = false end
	if CHB_Data.bHighlight == nil then CHB_Data.bHighlight = true end
end

-- ------------------------------------------------------------------------------------------------------

function CHB_OnEvent()
	
	if ( event == "VARIABLES_LOADED" ) then
		CHB_Init();

		CHB_Old_ChatFrame_OnEvent = ChatFrame_OnEvent;
		ChatFrame_OnEvent = CHB_ChatFrame_OnEvent;
		
		if GFF_AnalyseText then
			CHB_Supplement_Words();
			Print("Integrated Group Finder with ChanBuddy", 1.0, 0.5, 0.0);
		end
		
	elseif event == "CHAT_MSG_CHANNEL" then

		local msg, plr = arg1,arg2;
		local isspam;
		
		--Print(arg9);
		
		-- Monitor LFG channel just in case
		if (event=="CHAT_MSG_CHANNEL") and (arg9 == LFG_CHANNEL_NAME or string.upper(arg9) == "LFG") then
			--Print(arg9);
			arg1, isspam = CHB_FilterMessage(arg1, true);
			
			if not isspam then
				GFF_AnalyseText(msg, plr);
			end
			
		
		-- Support for Call To Arms
		-- Delayed until people actually use it exclusively
		-- For now it seems CTA users dump the LookingForGroup channel in the CTAChannel
		elseif (event=="CHAT_MSG_CHANNEL") and (arg9=="CTAChannel") then
			-- See if dumped messages might be usefull
			-- Disabled in favor of performance
--~ 			if string.find(arg1,"/cta [CD]") then
--~ 				local h,m,name,msg = string.gfind(arg1, "/cta [CD]<(%d%d)(%d%d):([^:]+):(.*):.*>")();
--~ 				if name and msg then
--~ 					GFF_AnalyseText(msg,name,true,h,m, plr);
--~ 				end
			--[[else]]if string.find(arg1,"/cta B") then				
				local h,m,msg,ver = string.gfind(arg1, "/cta B<(%d%d)(%d%d):(.*):(.*)>")();
				if msg and ver and h and m then
					GFF_AnalyseText("LFG " .. msg .. " (CTA " .. ver .. ")",plr,true,h,m, plr, true);
				end
			elseif string.find(arg1,"/cta A") then			
				--Print("CTA A");
				-- future work: parse /cta A messages more if need be
				local hour,mins,class,size,maxsize,minlevel,msg,opts = string.gfind(arg1, "/cta A<(%d%d)(%d%d)(%d%d%d)(%d%d)(%d%d)(%d%d):(.*):(.*)>")();
				local addinfo = ""; size = tonumber(size); maxsize = tonumber(maxsize);
				--pxd({hour=hour,mins=mins,class=class,size=size,maxsize=maxsize,minlevel=minlevel,msg=msg,opts=opts});
				-- Maybe trusted if it's A?
				if size and maxsize then
					if size > 40 then size = size - 40; addinfo = " (PVP)" end
					if maxsize > 40 then maxsize = maxsize - 40; end
					if msg and hour and mins then
						GFF_AnalyseText("LF" .. maxsize - size .. "M " .. msg .. " whisper for more information".. addinfo .. " (CTA)",plr,true,hour,mins, plr,true);
					end
				end
			end

		end

	end
end

-- ------------------------------------------------------------------------------------------------------

function CHB_nocase(s)
	s = string.gsub(s, "%a", function (c)
			return string.format("[%s%s]", string.lower(c), string.upper(c))
		end)
	  return s
end

local CHB_TEXT_COLORS = { chb = "0000ff7f", red = "00ff0000", 
					green = "0000ff00", blue = "000000ff", yellow = "00ffff00" };
					
function CHB_repcolor(s)
	return string.gsub(s, "|(%a*)`", function (c)
			return "|" .. CHB_TEXT_COLORS[c]
		end)
end

-- ------------------------------------------------------------------------------------------------------

function CHB_FilterMessage(msg, proc)
	local matchCount=0;
	local ret=nil;
	local suc = false;
	local isspam = false;
	
	for key,value in CHB_Data.words do
		suc = true; -- DB contains entries

		-- stringwide filter starts with meta &
		-- stringwide filter # is not usable in this context
		if string.sub(key,1,1) == "&" then
			if string.find(msg,string.sub(key,2)) then
				matchCount=matchCount+1;
			end
		elseif string.sub(key,1,1) == "#" then
			-- skip
		else
			for word in string.gfind(msg, "([^ ~,:;|<>/\\_!%(%)%[%]%*%-%?%.]+)") do
				if string.find(string.upper(word),"^" .. key .. "$") then
					matchCount=matchCount+1;

					if CHB_Data.bHighlight and not proc then
						msg = string.gsub(msg, word, "|c0000ff7f" .. word .. "|r", 1);
					else
						break; -- optimise here
					end					
				end
			end
		end
	end
	
	if not suc then
		-- db was empty
		return msg, false;
	end
	
	if (matchCount>0) then
		ret=msg;
	else
		ret = "*SPAM* " .. msg;
		isspam = true;
	end
	
	--Print("SPM 1='" .. arg1 .. "' 2=" .. arg2 .. "' 3='" .. arg3 .. "' 4='" .. arg4 .. "' 5='" .. arg5 .. "' 6='" .. arg6 .. "' 7='" .. arg7 .. "' 8='" .. arg8 .. "' 9='" .. arg9);	
	return ret, isspam;
end

-- ------------------------------------------------------------------------------------------------------

function CHB_ChatFrame_OnEvent(event)
	local isspam;
	local canpass = true;
	
	if CHB_Data.bFilter then
		-- Monitor LFG channel just in case
		if (event=="CHAT_MSG_CHANNEL") and (arg9 == LFG_CHANNEL_NAME or string.upper(arg9) == "LFG") then
			arg1, isspam = CHB_FilterMessage(arg1);
			canpass = not (isspam and CHB_Data.bHide);		
		end
	end
	
	if canpass then
		CHB_Old_ChatFrame_OnEvent(event);
	end
end;

-- ------------------------------------------------------------------------------------------------------
