﻿-- ----------------------------------------------------------------------- --
--     Group Finder with Chan Buddy                                        
--     Copyright (c) 2006 Moncai                                           
--
--     May be copied and distributed freely as long as you do not
--     change or remove this notice.
--     
-- ----------------------------------------------------------------------- --

GFF_FAUX_LINES = 20;
GFF_EXPIRE_LIMIT = 10;

-- --------------------------------------------------------------------------------------
-- Debugging functions
-- --------------------------------------------------------------------------------------

local DEBUG = false;

local dbg;

local function setdbg()
	if DEBUG then
		dbg = function (fun, ...)
			fun(unpack(arg));
		end
	else
		dbg = function() end
	end
end
setdbg();

function GFF_Debug()
	DEBUG = not DEBUG;
	setdbg();
end

-- --------------------------------------------------------------------------------------
-- Utility functions
-- --------------------------------------------------------------------------------------

GFF_H, GFF_M = 0, 0;

function GFF_TIME()
	-- GFF_H, GFF_M = date("%H"), date("%M");
	GFF_H, GFF_M = GetGameTime();
end	

local function HasChannel(channel)
	for a, b in {GetChannelList()} do
		if b == channel then
			return true;
		end
	end
	return false;
end

local function ChannelNumber(channel)
	local t, ret = {GetChannelList()}, "?";
	for i = 2, table.getn(t), 2 do
		if t[i] == channel then
			ret = tostring(t[i-1]);
		end
	end
	return ret;
end

function GFF_Check_LFG()
	-- If we don't have the LFG channel, join silently
	if not HasChannel(LFG_CHANNEL_NAME) then
		dbg(Print,string.format("Silently joining channel %s", LFG_CHANNEL_NAME));
		JoinChannelByName(LFG_CHANNEL_NAME);
		ChatFrame_RemoveChannel(DEFAULT_CHAT_FRAME, LFG_CHANNEL_NAME);
	end
end

function ToggleGroupFinder()
	if ( GroupFinderFrame:IsVisible() ) then
		HideUIPanel(GroupFinderFrame);
	else
		ShowUIPanel(GroupFinderFrame);
	end
end

function SetGroupFinderFrameSize(wantsmall)
	local f = getglobal("GroupFinderFrame");
	local t1,t2 = getglobal("GroupFinderFrameTopTexture1"),getglobal("GroupFinderFrameTopTexture2");
	local sc = getglobal("GroupFinderListScrollFrame");
	if wantsmall then
		f:SetHeight(512);
		t1:SetTexture("Interface\\QuestFrame\\UI-QuestLog-TopLeft");
		t2:SetTexture("Interface\\QuestFrame\\UI-QuestLog-TopRight");
		t1:SetHeight(256);
		t2:SetHeight(256);
		sc:SetHeight(93);
		GFF_FAUX_LINES = 6;
	else
		f:SetHeight(729);
		t1:SetTexture("Interface\\AddOns\\ChanBuddy\\graphics\\UI-QuestLog-TopLeft2");
		t2:SetTexture("Interface\\AddOns\\ChanBuddy\\graphics\\UI-QuestLog-TopRight2");
		t1:SetHeight(473);
		t2:SetHeight(473);
		sc:SetHeight(310);
		GFF_FAUX_LINES = 20;
	end
	
	GroupFinder_Update();
	
	local b = GroupFinderListScrollFrame:IsShown();
	GroupFinderListScrollFrame:Show();
	GroupFinderDetailScrollFrame:SetPoint("TOPLEFT", GroupFinderListScrollFrame, "BOTTOMLEFT", 0, -7);
	if not b then
		GroupFinderListScrollFrame:Hide();
	end
	
	GroupFinderDetailScrollFrame:UpdateScrollChildRect();
end

function SHOWMEM()
	local before, th = gcinfo();
	collectgarbage();
	local after = gcinfo();
	
	Print(string.format("Addon memory usage: current %.3f MiB, peak %.3f MiB, threshold %.3f MiB", after/1024, before/1024, th/1024), 1.0,0.8,0.0);
end

local g_name, g_level, g_class, g_race, g_guild;

function GFF_WhoResult(name, level, guild, race, class, lastSeen )
	if string.lower(g_name) == string.lower(name) then
		g_level = level; g_class = class; g_race = race; g_guild = guild;
	end
end

function GFF_plink(name) 
	return "|Hplayer:" .. name .. "|h" .. "[" .. name .. "]" .. "|h";
end

function GFF_Who(msg, ui, priority)
	SetWhoToUI(ui);
	SendWho("n-" .. msg, priority);
end

local function e(x)
	if x == nil then
		return "";
	else
		return x;
	end
end
local function n(x)
	if x == nil then
		return 0;
	else
		return x;
	end
end

function GFF_AddPerson(instance, text, name, untrusted, htime, mtime, source, isAtype) 
	local id;
	GFF_TIME();
	local hours, minutes = GFF_H, GFF_M;
	local rhours, rminutes = hours, minutes;
	local class, level, guild, race = GFF_UNKNOWN, "?", "", GFF_UNKNOWN;
	local tag;
	
	if untrusted then
		hours, minutes = htime, mtime;
		rhours, rminutes = htime, mtime;
	end
	
	if GFF.plookup[name] then
		if GFF.plookup[name].ins[instance] then
			id = GFF.plookup[name].ins[instance];

			if not GFF.data[instance].people[id] then
				Print("ADDPERSON ERROR i=" .. instance .. " id=" .. id, 1.0, 0.0, 0.0);
			end
			
			if (untrusted and not GFF.data[instance].people[id].untrusted) and not isAtype then
				dbg(Print, "Person " .. name .. " not updated: source " .. source .. " not trusted", 1,0.3,0);
				return;
			elseif (untrusted and not GFF.data[instance].people[id].untrusted) and isAtype then
				dbg(Print, "Person " .. name .. " updated: source " .. source .. " was atype", 1,0.3,0);
				text = text .. "\n" .. GFF.data[instance].people[id].text;
			end

			hours = GFF.data[instance].people[id].hours;
			minutes = GFF.data[instance].people[id].minutes;
			class = GFF.data[instance].people[id].class;
			level = GFF.data[instance].people[id].level;
			guild = GFF.data[instance].people[id].guild;
			race = GFF.data[instance].people[id].race;
		else
			GFF.data[instance].people.num = GFF.data[instance].people.num + 1;
			id = GFF.data[instance].people.num;			
			GFF.plookup[name].ins[instance] = id;
		end
	else
		GFF.data[instance].people.num = GFF.data[instance].people.num + 1;
		id = GFF.data[instance].people.num;
		GFF.plookup[name] = { who = 0, ins = { [instance] = id } }
		
		if GFF.data[instance].details.watched == 1 then
			if not PlaySoundFile("Interface\\AddOns\\ChanBuddy\\snd\\Message.wav") then
				PlaySoundFile("Interface\\AddOns\\ChanBuddy\\snd\\Hello.wav");
			end
			Print(string.format(GFF_NEW_WATCHED_PLAYER, GFF.data[instance].name, name), 1,0.8,0);
			GFF.data[instance].details.collapsed = 0;
		end
	end
	
	tag = GFF_LFGMX(text);
		
	GFF.data[instance].people[id] = {
		["name"] = name,
		["text"] = text,
		["cat"] = instance,
		["id"] = id,
		["class"] = class,
		["level"] = level,
		["guild"] = guild,
		["race"] = race,
		["hours"] = hours,
		["minutes"] = minutes,
		["rhours"] = rhours,
		["rminutes"] = rminutes,
		["tag"] = tag,
		["untrusted"] = untrusted,
		["atype"] = isAtype,
	};

	if untrusted then 
		dbg(Print, "GFF: Added person " .. name .. " from untrusted source " .. source, 1.0,0,0);
		GFF.data[instance].people[id].tag = "*" .. GFF.data[instance].people[id].tag;
	end
	
	if GFF.data[instance].people[id].level == "?" then
		
		GFF_Who(name, 1, MWTCE.PRIORITY.NORMAL);
	
		if CensusPlus_Database then
			g_name = GFF.data[instance].people[id].name; g_level = "?"; g_class = "Unknown"; g_race = "Unknown"; g_guild = "";
			
			local realmName = g_CensusPlusLocale .. GetCVar("realmName");
			CensusPlus_ForAllCharacters( realmName, UnitFactionGroup("player"), nil, nil, nil, nil, GFF_WhoResult);
			GFF.data[instance].people[id].class = e(g_class); GFF.data[instance].people[id].level = n(g_level); GFF.data[instance].people[id].race = e(g_race); GFF.data[instance].people[id].guild = e(g_guild);
		end
	end
end

local totnum = 0;

function GFF_AddItem(text, details)
	GFF.n = GFF.n + 1;
	GFF.items[GFF.n] = { ["text"] = text, ["details"] = details };
	
	if details.header == 1 then
		totnum = totnum + GFF.data[details.id].people.num;
	end
end

function GFF_Item(idx, person)
	--dbg("Add to list " .. person.name);
	
	if GDD.db[GFF.world][person.name] and (person.level == "?" or GDD.db[GFF.world][person.name].level >= person.level) then
		person.race = GDD.db[GFF.world][person.name].race;
		person.class = GDD.db[GFF.world][person.name].class;
		person.guild = GDD.db[GFF.world][person.name].guild;
		person.level = GDD.db[GFF.world][person.name].level;		
	end
	local text = person.name;
	GFF_AddItem(text, person);	
end

function GFF_RenderData()
	GFF_Empty()
	totnum = 0;
	GFF_TIME();
	local m, gm = GFF_H*60 +  GFF_M, 0;
	
	-- Cleanup
	for i = 1, GFF.data.num, 1 do
		local id = 1;
		while id <= GFF.data[i].people.num do
			if not GFF.data[i].people[id] then
				Print(string.format("RENDERDATA ERROR GFF.data[%d].people[%d] people.num=%d ", i, id, GFF.data[i].people.num), 1.0, 0.3, 0.3);
			end

			gm = m - (GFF.data[i].people[id].rhours * 60 + GFF.data[i].people[id].rminutes);
			if gm < 0 then
				-- tricky around midnight
				gm = gm + (24*60);
			end
			
			if (gm > GFF_EXPIRE_LIMIT) or (GFF.data[i].details.prune and GFF.data[i].details.prune(GFF.data[i].people[id])) then
				dbg(Print,string.format("Expired: %s data[%d].people[%d] (%s)", GFF_plink(GFF.data[i].people[id].name), i, id, GFF.data[i].name), 0.5, 0.0, 1.0);
				
				if GFF.plookup[GFF.data[i].people[id].name] and GFF.plookup[GFF.data[i].people[id].name].ins then
					GFF.plookup[GFF.data[i].people[id].name].ins[i]= nil;
				end

				-- remove
				GFF.data[i].people[id] = nil;
				-- renumber
				for k = id, GFF.data[i].people.num-1, 1 do
					GFF.data[i].people[k] = GFF.data[i].people[k+1];
					GFF.data[i].people[k].id = k;
				end
				GFF.data[i].people[GFF.data[i].people.num] = nil;
				GFF.data[i].people.num = GFF.data[i].people.num - 1;
			else
				id = id + 1;
			end
		end
		
		for id = 1, GFF.data[i].people.num, 1 do
			-- resequence id's
			GFF.data[i].people[id].id = id;
			GFF.plookup[GFF.data[i].people[id].name].ins[i] = id
		end
		
	end
	
	for i = 1, GFF.data.num, 1 do
		if GFF.showempty or GFF.data[i].people.num > 0 then
			GFF_AddItem(GFF.data[i].name, GFF.data[i].details);
			if GFF.data[i].details.collapsed == 0 then
				for k = 1, GFF.data[i].people.num, 1 do
					GFF_Item(k, GFF.data[i].people[k]);
				end
			end
		end
	end
	
	
end

function  GFF_Empty()
	GFF.n = 0;
	GFF.items = {};
end

function GFF_AnalyseText(msg, person, untrusted, htime, mtime, source, isAtype)
	-- pre: msg is not spam
	local settled = false;
	
	if not msg or not person then
		if not trusted then
			dbg(Print, "Received empty untrusted message");
		end
		return;
	end
	
	for idx = 1, GFF.data.num, 1 do
		local found = false;
		for k, key in GFF.data[idx].keys do
			if string.sub(key,1,1) == "&" then
				if string.find(string.upper(msg),string.sub(key,2)) then
					dbg(Print, string.format("found key `%s` in message %s:`%s` from `person`", key, person, msg));
					found = true;
				end
			elseif string.sub(key,1,1) == "#" then
				if string.find(string.upper(msg),string.sub(key,2)) then
					dbg(Print, string.format("found key `%s` in message %s:`%s`, aborting", key, person, msg));
					found = false;
					break;
				end
			else
				for word in string.gfind(msg, GFF_WORD_SPLIT) do
					if string.find(string.upper(word),"^" .. key .. "$") then
						dbg(Print, string.format("found key `%s` matching word `%s` in message %s:`%s`", key, word, person, msg));
						found = true;
						break;
					end
				end
				
--~				-- Disabled optimise to support #
--~ 				if found then
--~ 					break
--~ 				end
			end
		end
		
		if found then
			GFF_AddPerson(idx, msg, person, untrusted, htime, mtime, source, isAtype);
			settled = true;
		end
	
	end
	
	if not settled then
		GFF_AddPerson(GFF.data.num, msg, person, untrusted, htime, mtime, source, isAtype);
	end
	
	GroupFinder_Update();
end

-- --------------------------------------------------------------------------------------
-- Button functions
-- --------------------------------------------------------------------------------------

function GroupFinderTitleButton_OnLoad()
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp");
end

local oc = 1;
local op = 1;

function GroupFinderTitleButton_OnEvent(event)

end

function GroupFinderTitleButton_OnClick(button)
	local itemindex = this:GetID() + FauxScrollFrame_GetOffset(GroupFinderListScrollFrame);
	GroupFinder_SetSelection(itemindex);
	
	GroupFinder_Update();
end

function GroupFinderTitleButton_OnEnter()
	-- Set highlight
	getglobal(this:GetName().."Tag"):SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
end

function GroupFinderAbandonButton_OnClick()
	if (not GFF.selname) or GFF.selname == "~" then 
		return;
	end
	
	local name = GFF.selname;

	if ( not ChatFrameEditBox:IsVisible() ) then
		ChatFrame_OpenChat("/w "..name.." ");
	else
		ChatFrameEditBox:SetText("/w "..name.." ");
	end
	ChatEdit_ParseText(ChatFrame1.editBox, 0);
end

function GFF_WhoPush()
	if (not GFF.selname) or GFF.selname == "~" then 
		return;
	end
	
	local name = GFF.selname;
		
	GFF.whodisplay = GFF.whodisplay + 1;
	
	GFF_Who(name, 1, MWTCE.PRIORITY.HIGHEST);
end

function GFF_Invite()
	if (not GFF.selname) or GFF.selname == "~" then 
		return;
	end
	
	local name = GFF.selname;

	InviteByName(name);
end

-- --------------------------------------------------------------------------------------
-- frame functions
-- --------------------------------------------------------------------------------------

function GroupFinder_OnLoad()
	this.selectedButtonID = 2;
	this:RegisterEvent("WHO_LIST_UPDATE");
	this:RegisterEvent("VARIABLES_LOADED");

	if(not Print) then
		function Print(msg, r, g, b, frame) 
			if (not r) then r = 1.0; end
			if (not g) then g = 1.0; end
			if (not b) then b = 1.0; end
			if ( frame ) then 
				frame:AddMessage(msg,r,g,b);
			else
				if ( DEFAULT_CHAT_FRAME ) then 
					DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
				end
			end
		end
	end

	if (not PrintEx) then
		function PrintEx(msg, col, frame) 
			Print(msg, col.r, col.g, col.b, frame);
		end
	end

	-- Initialize character database
	GFF.world = GetLocale() .. "-" .. GetRealmName();
		
	SlashCmdList["LFGCOMMAND"] = GroupFinder_Slash;
	SLASH_LFGCOMMAND1 = "/lfg";
	SLASH_LFGCOMMAND2 = "/gfind";

	SlashCmdList["CWHOCOMMAND"] = GroupFinder_Slash_Who;
	SLASH_CWHOCOMMAND1 = "/cwho";

	SlashCmdList["GFFRCOMMAND"] = GroupFinder_Slash_Resize;
	SLASH_GFFRCOMMAND1 = "/lfgresize";
	SLASH_GFFRCOMMAND2 = "/lfgr";

	SlashCmdList["GFFJOINCOMMAND"] = function(msg)
		JoinChannelByName(LFG_CHANNEL_NAME);
		ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, LFG_CHANNEL_NAME);
		Print(string.format(CHAT_YOU_JOINED_NOTICE, ChannelNumber(LFG_CHANNEL_NAME) .. ". " .. LFG_CHANNEL_NAME), 1, 0.8, 0);
	end
	SLASH_GFFJOINCOMMAND1 = "/lfgjoin";

	SlashCmdList["GFFLEAVECOMMAND"] = function(msg)		
		for i = 1, NUM_CHAT_WINDOWS, 1 do			
			ChatFrame_RemoveChannel(getglobal("ChatFrame" .. i), LFG_CHANNEL_NAME);
		end
		Print(string.format(CHAT_YOU_LEFT_NOTICE, ChannelNumber(LFG_CHANNEL_NAME) .. ". " .. LFG_CHANNEL_NAME), 1, 0.8, 0);
	end
	SLASH_GFFLEAVECOMMAND1 = "/lfgleave";
	
	GroupFinderQuestTitle:SetText("Empty");
	GroupFinderQuestDescription:SetText("");
	
	GroupFinderRewardTitleText:Hide();
	
	-- try to integrate with MWTCE if possible
	
	-- Hook no longer needed with MWTCE
	--Hook chat frame to prevent it from showing all the time
	--GFF.FriendsFrame_OnEvent = FriendsFrame_OnEvent;
	--FriendsFrame_OnEvent = GFF_FriendsFrame_OnEvent;
	
	GFF.FriendsFrameFriendButton_OnClick = FriendsFrameFriendButton_OnClick;
	FriendsFrameFriendButton_OnClick = GFF_FriendsFrameFriendButton_OnClick;
	
  	Print(GFF_LOADED_MESSAGE, 1.0, 0.5, 0.0);
	
	UIErrorsFrame:AddMessage(GFF_ENABLED_MESSAGE, 1.0, 0.5, 0.0, 1.0, UIERRORS_HOLD_TIME);
	
end

function GroupFinder_Slash(msg)
	ToggleGroupFinder();
end

function GroupFinder_Slash_Resize(msg)
	if not GDD.wantsmall then
		GDD.wantsmall = true;
	else
		GDD.wantsmall = nil;
	end
	SetGroupFinderFrameSize(GDD.wantsmall);
	
	if GDD.wantsmall then
		Print(GFF_RESIZE_S);
	else
		Print(GFF_RESIZE_L);
	end
end

function GroupFinder_Slash_Who(msg)
	local n = 0;
	msg = string.lower(msg);
	
	for name, details in GDD.db[GFF.world] do
		if string.find(string.lower(name),msg) then
			n = n + 1;
			local g = details.guild; if g ~= "" then g = " <" .. g .. ">"; end
			Print(format(GFF_WHO_RESULT, GFF_plink(name), details.race, details.class, details.level, g), 1.0, 0.8, 0.0);
			
			GFF_Who(name, 1, MWTCE.PRIORITY.LOW);
		end
	end
	
	if n == 0 then
		GFF.whodisplay = GFF.whodisplay + 1;
		GFF_Who(msg, 1, MWTCE.PRIORITY.HIGHEST);
	end
end

function GroupFinder_OnEvent(event)
	if event == "WHO_LIST_UPDATE" then 
		-- process who's we got :)

		if not GDD then
			GDD = {}; GDD.db = {};
			GDD.db[GFF.world] = {};
		end
		
		for i = 1, GetNumWhoResults(), 1 do
			local charname, guildname, level, race, class, zone = GetWhoInfo(i);
			if GFF.whodisplay > 0 then
				local g = guildname; if g ~= "" then g = " <" .. g .. ">"; end

				Print(format(GFF_WHO_RESULT_ZONE, GFF_plink(charname), race, class, level, g, zone), 1.0, 0.8, 0.0);
			end
			
			GDD.db[GFF.world][charname] = {
				["guild"] = guildname,
				["level"] = level,
				["race"] = race,
				["class"] = class,
			};

		end
		if GFF.whodisplay > 0 then GFF.whodisplay = GFF.whodisplay - 1; end
		GroupFinder_Update();

	elseif event == "VARIABLES_LOADED" then		
		-- Do data upgrades here
		SetGroupFinderFrameSize(GDD.wantsmall);
		
		if not GDD.db then
			Print(GFF_INIT_DB);
			GDD.db = {};
		end
		
		if not GDD.db[GFF.world] then
			GDD.db[GFF.world] = {};			
			if GDD.people then
				Print(GFF_CONVERT_DB);
				GDD.db[GFF.world] = GDD.people;
				GDD["people"] = nil;
			end
		end	
		
		GFF_Check_LFG();
		
		Print(string.format(GFF_BETA_NOTICE, GROUP_FINDER), 0.3,0.7,0.3);
		
	end
end

function GroupFinder_OnShow()
	PlaySound("igQuestLogOpen");
	GroupFinder_Update();
end

function GroupFinder_OnHide()
	PlaySound("igQuestLogClose");
end

local phox, upstep = 0, 23;
local meandar, step = 0, 5;

function GroupFinder_OnUpdate(elapsed)
	phox = phox + elapsed;
	
	if phox >= upstep then
		phox = phox - upstep;
		GroupFinder_Update();
	end
	
	meandar = meandar + elapsed;
	if meandar >= step then
		meandar = meandar - step;
		GFF_Check_LFG();
	end
end

function GroupFinder_Update()
	local n,d;
	GFF_TIME();
	local m,gm = GFF_H*60 +  GFF_M,0;

	-- Set the expand/collapse all button texture
	local numHeaders = 0;
	local notExpanded = 0;	
	
	GFF_RenderData();
	n = 0;
	
	-- ScrollFrame update
	FauxScrollFrame_Update(GroupFinderListScrollFrame, GFF.n, GFF_FAUX_LINES, QUESTLOG_QUEST_HEIGHT, nil, nil, nil, GroupFinderHighlightFrame, 293, 316);
	
	-- Update the listing
	GroupFinderHighlightFrame:Hide();
	
	local t,ttag,thl,thm,thc, textwidth;
	
	for i=1, GFF_FAUX_LINES, 1 do
		t = getglobal("GroupFinderTitle"..i);
		ttag = getglobal("GroupFinderTitle"..i.."Tag");
		thl = getglobal("GroupFinderTitle"..i.."Highlight");
		thm = getglobal("GroupFinderTitle"..i.."GroupMates");
		thc = getglobal("GroupFinderTitle"..i.."Check");

		local text = "";
		
		local id = i + FauxScrollFrame_GetOffset(GroupFinderListScrollFrame);
		if id > GFF.n then
			t:Hide();
		else
			d = GFF.items[id];
			
			if d.details["header"] == 1 then
				numHeaders = numHeaders + 1;
				
				local htext = d.text .. " (" .. GFF.data[d.details.id].people.num .. ")"
				t:SetText(htext);
				
				ttag:SetText(d.details.levelrange);
				thm:SetText("");
				
				if d.details.collapsed == 1 then
					t:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
					notExpanded = notExpanded + 1;
				else
					t:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up"); 
				end
				thl:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
				
				if d.details.watched == 1 then					
					GroupFinderDummyText:SetText(htext);
					thc:SetPoint("LEFT", t, "LEFT",  GroupFinderDummyText:GetWidth() +24, 0);
					thc:Show();
				else
					thc:Hide();
				end
			else
				thc:Hide();
				
				n = n + 1;
				
				gm = m-(d.details.rhours*60 + d.details.rminutes);
				if gm < 0 then gm = gm + 24*60; end

				if ( gm < 2 ) then
					thm:SetText("*");
				else
					thm:SetText("");
				end
				
				if d.details.tag and d.details.tag ~= "" then
					text = d.text .. " " .. d.details.tag;
				else
					text = d.text;
				end

				t:SetText(string.format(GFF_HEADER_LINE, text, gm));
				t:SetNormalTexture("");
				thl:SetTexture("");
				ttag:SetText(d.details.level .. " " .. d.details.class);
			end
			
			if (d.details["header"] == 1 ) then
				color = QuestDifficultyColor["header"];
			else
				if not d.details.class or d.details.class == GFF_UNKNOWN then
					color = NORMAL_FONT_COLOR;
				else
					-- Need fix here for localized clients
					if not RAID_CLASS_COLORS[string.upper(d.details.class)] then
						if GFF_CLASS_LOC[d.details.class] then
							color = RAID_CLASS_COLORS[GFF_CLASS_LOC[d.details.class]];
						else
							color = NORMAL_FONT_COLOR;
						end
					else
						color = RAID_CLASS_COLORS[string.upper(d.details.class)];
					end
				end				
			end

			ttag:SetTextColor(color.r, color.g, color.b);
			t:SetTextColor(color.r, color.g, color.b);
			-- questNumGroupMates:SetTextColor(color.r, color.g, color.b);
			t.r = color.r;
			t.g = color.g;
			t.b = color.b;
			t:Show();
			
			if ( GroupFinderFrame.selectedButtonID and GFF.selected == id and GFF.selname == d.details.name ) then
				GroupFinderHighlightFrame:SetPoint("TOPLEFT", "GroupFinderTitle"..i, "TOPLEFT", 0, 0);
				GroupFinderHighlightFrame:Show();
				ttag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
				t:LockHighlight();
			else
				t:UnlockHighlight();
			end
			
		end	
	end
	
	for i = GFF_FAUX_LINES + 1, 20, 1 do
		local t = getglobal("GroupFinderTitle"..i);
		t:Hide();
	end
	
	-- If all headers are not expanded then show collapse button, otherwise show the expand button
	if ( notExpanded ~= numHeaders ) then
		GroupFinderCollapseAllButton.collapsed = nil;
		GroupFinderCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
	else
		GroupFinderCollapseAllButton.collapsed = 1;
		GroupFinderCollapseAllButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
	end
	
	GroupFinderQuestCount:SetText(totnum .. GFF_XLINES);
	
	if (GFMB_UpdateMinimapIcon) then
		GFMB_UpdateMinimapIcon(totnum);
	end
end

function GroupFinder_SetSelection(id)
	local bid = id - FauxScrollFrame_GetOffset(GroupFinderListScrollFrame);

	local titleButton = getglobal("GroupFinderTitle"..bid);
	local titleButtonTag = getglobal("GroupFinderTitle"..bid.."Tag");
	local item = GFF.items[id];
		
	if ( item.details["header"] == 1 ) then
		if IsControlKeyDown() and not IsShiftKeyDown() then
			for i = 1, GFF.data.num, 1 do
				GFF.data[i].details.watched = 1;
			end
		elseif IsShiftKeyDown() then
			if IsControlKeyDown() then
				for i = 1, GFF.data.num, 1 do
					GFF.data[i].details.watched = 0;
				end
			else
				if GFF.data[item.details.id].details.watched == 1 then
					GFF.data[item.details.id].details.watched = 0;
				else
					GFF.data[item.details.id].details.watched = 1;
				end
			end
			return;
		else
			if ( item.details.collapsed == 1) then
				GFF.data[item.details.id].details.collapsed = 0;
			else
				GFF.data[item.details.id].details.collapsed = 1;
			end
		end
		
	else
		-- Set newly selected ad and highlight it
		GFF.selected = id;
		GFF.selname = item.details.name;
		
		GroupFinderFrame.selectedButtonID = id;
		
		local scrollFrameOffset = FauxScrollFrame_GetOffset(GroupFinderListScrollFrame);
		if ( id > scrollFrameOffset and id <= (scrollFrameOffset + GFF_FAUX_LINES) and id <= GFF.n ) then
			titleButton:LockHighlight();
			titleButtonTag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			GroupFinderSkillHighlight:SetVertexColor(titleButton:GetTextColor());
			GroupFinderHighlightFrame:SetPoint("TOPLEFT", "GroupFinderTitle"..bid, "TOPLEFT", 5, 0);
			GroupFinderHighlightFrame:Show();		
		end

		-- Update the details
		GroupFinder_UpdateQuestDetails();
	end	
end

function GroupFinder_UpdateQuestDetails(doNotScroll)
	
	local id = GFF.selected;
	local d = GFF.items[id];
	
	if not d then
		return;
	end
	
	local p = GFF.data[d.details.cat];
	GFF_TIME();
	local hr, mins = GFF_H, GFF_M;
	mins = hr*60 + mins;
	
	GroupFinderObjective1:SetText(string.format(GFF_OBJ_LINES1, d.details.name,d.details.level,d.details.race,d.details.class));
	GroupFinderObjective1:SetTextColor(0, 0, 0);
	GroupFinderObjective1:Show();
	
	GroupFinderObjective2:SetText(string.format(GFF_OBJ_LINES2, d.details.hours, d.details.minutes, mins-(d.details.hours*60 + d.details.minutes)));
	GroupFinderObjective2:SetTextColor(0, 0, 0);
	GroupFinderObjective2:Show();
	
	GroupFinderObjective3:SetText(string.format(GFF_OBJ_LINES3, d.details.rhours, d.details.rminutes, mins-(d.details.rhours*60 + d.details.rminutes)));
	GroupFinderObjective3:SetTextColor(0, 0, 0);
	GroupFinderObjective3:Show();
	
	GroupFinderQuestTitle:SetText(p.name);
	GroupFinderQuestDescription:SetText(d.details.text .. string.format(GFF_SINGATURE, d.details.name, d.details.guild));
	
	GroupFinderRewardTitleText:Hide();
	
	GroupFinderDetailScrollFrame:UpdateScrollChildRect();
end

--Used to attach an empty spacer frame to the last shown object
function GroupFinderFrame_SetAsLastShown(frame, spacerFrame)
	if ( not spacerFrame ) then
		spacerFrame = GroupFinderSpacerFrame;
	end
	spacerFrame:SetPoint("TOP", frame, "BOTTOM", 0, 0);
end

function GroupFinderCollapseAllButton_OnClick()	
	if (this.collapsed) then
		this.collapsed = nil;
		for i = 1, GFF.data.num, 1 do
			GFF.data[i].details.collapsed = 0;
		end
	else
		this.collapsed = 1;
		QuestLogListScrollFrameScrollBar:SetValue(0);
		for i = 1, GFF.data.num, 1 do
			GFF.data[i].details.collapsed = 1;
		end
	end
	
	GroupFinder_Update();
end

---------------------------------------------------------------------------
-- Hooked functions
---------------------------------------------------------------------------

function GFF_FriendsFrame_OnEvent()
	local maypass = true;

	if event == "WHO_LIST_UPDATE" then
		maypass = GFF.whoframe;
		GroupFinder_OnEvent(event);
	end
	
	if maypass then
		GFF.FriendsFrame_OnEvent();
	end
end

function GFF_FriendsFrameFriendButton_OnClick(button)
	GFF.FriendsFrameFriendButton_OnClick(button);
	
	if ( button == "LeftButton" and IsShiftKeyDown() ) then
		local name, level, class, area, connected = GetFriendInfo(this:GetID());
		if ( not ChatFrameEditBox:IsVisible() ) then
			ChatFrame_OpenChat(name);
		else
			ChatFrameEditBox:SetText(name);
		end
	end
end

-- --------------------------------------------------------------------------------------
