﻿-- ----------------------------------------------------------------------- --
--     Moncaí's Who Toolkit Embedded
--     Copyright (c) 2006 Moncaí
--
--     May be copied and distributed freely as long as you do not
--     change this file.
--     
-- ----------------------------------------------------------------------- --
local MWTCE_CURRENT_VERSION = 2.4;
local MWTCE_UNIQUE_ID = "MWTCE Embedded with Group Finder";
local DEBUG = false;
local MWTCE_IS_EMBEDDED = true;
local MWCTE_CLASH;

-------------------------------------------------------------
-- Debugging functions
-------------------------------------------------------------
local dbg;
local function setdbg()
	if DEBUG then
		dbg = function (fun, ...)
			fun(unpack(arg));
		end
	else
		dbg = function() end
	end
end
setdbg();

-------------------------------------------------------------
-- Embedding support
-------------------------------------------------------------

if not (MWTCE and type(MWTCE) == "table" and tonumber(MWTCE.version) and tonumber(MWTCE.version) >= MWTCE_CURRENT_VERSION) then

	if not MWTCE_VERSIONS then MWTCE_VERSIONS = {}; end
	MWTCE_VERSIONS[MWTCE_UNIQUE_ID .. tostring(MWTCE_CURRENT_VERSION)] = MWTCE_IS_EMBEDDED;

	if MWTCE and  type(MWTCE) == "table" and MWTCE.version then
		MWTCE_OVERRIDE = ("We already had MWTCE version " .. MWTCE.version .. ", overriding with " .. MWTCE_CURRENT_VERSION);
	end

	-------------------------------------------------------------
	-- The main class
	-------------------------------------------------------------	
	
	MWTCE = {
		embedded = MWTCE_IS_EMBEDDED,
		version = MWTCE_CURRENT_VERSION, 
		queue = {},
		hook = {},
		whotoUI = 0,
		ui_want_request = false, 
		timing = {
			lastwhotime = GetTime(),
			timedelta = 5.5,
			timestep = 0.2,
		},
		whoinfo = {
			n = 0,
			t = 0,
			results = {},
		},
		MOD = { CENSUS = 1, }, 
		override = {},
		compatibility = {},
		blizzard = { SetWhoToUI=SetWhoToUI, SendWho=SendWho },
		PRIORITY = { UI = 1, HIGHEST = 2, HIGH = 3, NORMAL = 4, LOW = 5, LOWEST = 6, },
		pstrings = { "|c0000ff00UI|r", "|c000000ffHIGHEST|r", "|c00ffff00HIGH|r", "|c00ffffffNORMAL|r", "|c00ff0000LOW|r", "|c00ccccccLOWEST|r"};
	};

	-------------------------------------------------------------
	-- All-purpose utility functions
	-------------------------------------------------------------
	
	if(not Print) then
		function Print(msg, r, g, b, frame) 
			if (not r) then r = 1.0; end
			if (not g) then g = 1.0; end
			if (not b) then b = 1.0; end
			if ( frame ) then 
				frame:AddMessage(msg,r,g,b);
			else
				if ( DEFAULT_CHAT_FRAME ) then 
					DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
				end
			end
		end
	end

	if (not PrintEx) then
		function PrintEx(msg, col, frame) 
			Print(msg, col.r, col.g, col.b, frame);
		end
	end	

	-- push at back
	function MWTCE.push(priority, msg, modflag)
		if not MWTCE.queue[priority] then MWTCE.queue[priority] = {}; end
		if not MWTCE.queue[priority].count then MWTCE.queue[priority].count = 0; end

		if MWTCE.queue[priority].last then
			MWTCE.queue[priority].last.next = { msg=msg, modflag=modflag, toUI=MWTCE.whotoUI, prev=MWTCE.queue[priority].last, next=nil };
			MWTCE.queue[priority].last = MWTCE.queue[priority].last.next;
		else
			MWTCE.queue[priority].first = { msg=msg, toUI=MWTCE.whotoUI, modflag=modflag, next = nil, prev = nil };
			MWTCE.queue[priority].last = MWTCE.queue[priority].first;
		end
		
		MWTCE.queue[priority].count = MWTCE.queue[priority].count + 1;
	end
	
	function MWTCE.Debug()
		DEBUG = not DEBUG;
		setdbg();
	end
	
	-- pop from front: as queue
	function MWTCE.popf(priority)
		local ret = nil;
		
		if not MWTCE.queue[priority] then MWTCE.queue[priority] = {}; end
		if not MWTCE.queue[priority].count then MWTCE.queue[priority].count = 0; end

		if MWTCE.queue[priority].first then
			ret = MWTCE.queue[priority].first;
			
			if MWTCE.queue[priority].first.next then
				MWTCE.queue[priority].first = MWTCE.queue[priority].first.next;
				MWTCE.queue[priority].first.prev = nil;
			else
				MWTCE.queue[priority].first = nil;
				MWTCE.queue[priority].last = MWTCE.queue[priority].first;
			end
			MWTCE.queue[priority].count = MWTCE.queue[priority].count - 1;
		end
		
		if ret then ret.prev = nil; ret.next = nil; end
		return ret;
	end

	-- pop from back: as stack
	function MWTCE.popb(priority)
		local ret;
		
		if not MWTCE.queue[priority] then MWTCE.queue[priority] = {}; end
		if not MWTCE.queue[priority].count then MWTCE.queue[priority].count = 0; end

		if MWTCE.queue[priority].last then
			ret = MWTCE.queue[priority].last;
			
			if MWTCE.queue[priority].last.prev then
				MWTCE.queue[priority].last = MWTCE.queue[priority].last.prev;
				MWTCE.queue[priority].last.next = nil;
			else
				MWTCE.queue[priority].first = nil;
				MWTCE.queue[priority].last = MWTCE.queue[priority].first;
			end
			MWTCE.queue[priority].count = MWTCE.queue[priority].count - 1;
		end
		
		if ret then ret.prev = nil; ret.next = nil; end
		return ret;
	end	
	
	-------------------------------------------------------------
	-- Who queueing functions
	-------------------------------------------------------------
	
	function MWTCE.ResolveWho(onrequest)
		local now = GetTime();
		local timetonextwho = MWTCE.timing.timedelta - (now - MWTCE.timing.lastwhotime);
		if timetonextwho < 0 then timetonextwho = 0; end
		
		if onrequest then
			dbg(Print, "direct request to Resolve");
			local qtime;
			if  MWTCE.queue and MWTCE.queue[MWTCE.PRIORITY.UI] and MWTCE.queue[MWTCE.PRIORITY.UI].count then
				qtime = (MWTCE.queue[MWTCE.PRIORITY.UI].count -1) * MWTCE.timing.timedelta;
			else
				qtime = 0.0;
			end
			if (qtime + timetonextwho) > 1.0 then
				Print(string.format(MWTCE.STR_WHO_DLY, qtime + timetonextwho), 1.0,0.5,0)
			end
		end
		
		if GFMB_IconMsg then dbg(GFMB_IconMsg, string.format("Timedelta %.1f, next who in %.1f", now - MWTCE.timing.lastwhotime, timetonextwho)); end
		
		if (now - MWTCE.timing.lastwhotime) >= MWTCE.timing.timedelta then
			local request = MWTCE.popf(MWTCE.PRIORITY.UI);			
			if request then
				dbg(Print, string.format("Sending Who request \"%s\", priority: %s", request.msg, MWTCE.pstrings[MWTCE.PRIORITY.UI]), 1.0, 0.5, 0.0);
				-- Process request to UI
				if WhoFrame:IsVisible() then MWTCE.hook.SetWhoToUI(1); else MWTCE.hook.SetWhoToUI(0); end
				MWTCE.ui_want_request = true;
				MWTCE.timing.lastwhotime = now;
				MWTCE.hook.SendWho(request.msg);
			else
				for k = MWTCE.PRIORITY.HIGHEST, MWTCE.PRIORITY.LOWEST,1 do
					request = MWTCE.popf(k);
					if request then
						dbg(Print, string.format("Sending Who request \"%s\", priority: %s", request.msg, MWTCE.pstrings[k]), 1, 0.5, 0);
						-- Process request to AddOns
						MWTCE.hook.SetWhoToUI(request.toUI); -- We can safely assume that an addon is responsible, but some addons want the UI to have it -- request.toUI
						MWTCE.ui_want_request = false;
						
						--if request.modflag then	Print("MODFLAG", 0,1,0) end
						
						MWTCE.modflag = request.modflag;
						MWTCE.timing.lastwhotime = now;
						MWTCE.hook.SendWho(request.msg);
						break;
					end
				end
			end
		end
	end
	
	function MWTCE.RegisterWho()
		MWTCE.hook.SendWho = SendWho;
		SendWho = MWTCE.SendWho;
		
		MWTCE.hook.SetWhoToUI = SetWhoToUI;
		SetWhoToUI = MWTCE.SetWhoToUI;

		MWTCE.hook.WhoFrameEditBox_OnEnterPressed = WhoFrameEditBox_OnEnterPressed;
		WhoFrameEditBox_OnEnterPressed = MWTCE.WhoFrameEditBox_OnEnterPressed;

		MWTCE.hook.SlashWho = SlashCmdList["WHO"];
		SlashCmdList["WHO"] = MWTCE.SlashWho;
	
		MWTCE.hook.FriendsFrame_OnEvent = FriendsFrame_OnEvent;
		FriendsFrame_OnEvent = MWTCE.FriendsFrame_OnEvent;	
		
		MWTCE.hook.WhoList_Update = WhoList_Update;
		WhoList_Update = MWTCE.WhoList_Update;	

 		MWTCE.hook.SortWho = SortWho;
 		SortWho = MWTCE.SortWho;	
		
--~ 		MWTCE.hook. = ;
--~ 		 = MWTCE.;	
	end
	
	-------------------------------------------------------------
	-- OnEvent handlers
	-------------------------------------------------------------
	
	function MWTCE.OnLoad()
		if MWTCE.addonisloaded then		
			if not MWTCE.compatibility.selfresolve then 
				MWTCE.compatibility.selfresolve = 0;
			end
			MWTCE.compatibility.selfresolve = MWTCE.compatibility.selfresolve + 1;
			
			dbg(Print, "MWTCE is already loaded so this frame is discarded");
			this:Hide();
			this:SetScript("OnEvent", nil);
			this:SetScript("OnUpdate", nil);
			return;
		end 
		MWTCE.addonisloaded = true;
	
		-- For timing purposes
		this:RegisterEvent("WHO_LIST_UPDATE");
		this:RegisterEvent("VARIABLES_LOADED");

		-- Might be needed to check correct installation
		-- this:RegisterEvent("VARIABLES_LOADED");
	end

	function MWTCE.OnEvent()
		local now = GetTime();

		if event == "WHO_LIST_UPDATE" then
			MWTCE.timing.lastwhotime = now;
		elseif event == "VARIABLES_LOADED" then
			-- Drastic measures to prevent other mods from interfering
			FriendsFrame:SetScript("OnEvent", function() --[[dbg(Print,"Direct hook FriendsFrame_OnEvent");]] MWTCE.FriendsFrame_OnEvent() end);

			if GFF_WHO_DELAYED ~= nil then
				MWTCE.STR_WHO_DLY = GFF_WHO_DELAYED 
			else
				MWTCE.STR_WHO_DLY = "* The results of your who will arrive in %.1f seconds."
			end

			-- Hook functions as late as possible
			MWTCE.RegisterWho();
			
			-- PlayerLinkMenu compatibility
			if PLAYERLINKMENU_VERSION and PLAYERLINKMENU_NAME then
				local WHO_ID = 3;
				plmActions[WHO_ID].action = function(name)
					SendWho("n-"..name, MWTCE.PRIORITY.UI);
				end
				Print(string.format("Integrated %s version %s with %s (%s)", PLAYERLINKMENU_NAME, PLAYERLINKMENU_VERSION, GROUP_FINDER, plmLabels[WHO_ID]),1,0.8,0);
			else
				--No PlayerLinkMenu? Then we want to rewrite SetItemRef
				MWTCE.hook.SetItemRef = SetItemRef;
				SetItemRef = MWTCE.SetItemRef;
			end
			
			-- AcePlayerMenu compatibility
			if AcePlayerMenu and AcePlayerMenu.SetItemRef and AcePlayerMenu.UnitPopup_OnClick then
				MWTCE.compatibility.AcePlayerMenu();
				Print(string.format("Integrated %s version %s with %s", AcePlayerMenu.name, AcePlayerMenu.version, GROUP_FINDER),1,0.8,0);
			end
			
			-- Chatr Enhancement compatibility
			if Chatr_Who then
				Print(string.format("Integrated %s with %s", "Chatr", GROUP_FINDER),1,0.8,0);
				function Chatr_Who(s)
					--FriendsFrame:UnregisterEvent("WHO_LIST_UPDATE");
					SetWhoToUI(1);
					SendWho(s, MWTCE.PRIORITY.HIGHEST);
					Chatr_Whoed=1;
				end
			end
		end
	end
	
	local meandar, limit = 0, 0.5;
	function MWTCE.OnUpdate(elapsed)
		meandar = meandar + elapsed;
		if meandar >= limit then
			--dbg(Print, string.format("tick %.3f %.3f", meandar, elapsed));
			meandar = meandar - limit;
			MWTCE.ResolveWho();
		end
	end

	-------------------------------------------------------------
	-- Override Blizzard Who functions
	-------------------------------------------------------------
	
	function MWTCE.SetWhoToUI(bEnabled)
		local ret = MWTCE.whotoUI;
		MWTCE.whotoUI = bEnabled;
		
		dbg(Print,"SetWhoToUI(" .. tostring(bEnabled) .. ")");
		-- Temporary transparancy
		--MWTCE.hook.SetWhoToUI(bEnabled);
		return ret;
	end
	
	function MWTCE.SendWho(msg, priority, modflag)
		if not priority then priority = MWTCE.PRIORITY.LOWEST; end
		dbg(Print, string.format("Queueing Who request \"%s\", priority: %s", msg, MWTCE.pstrings[priority]), 1.0, 0.8, 0.0);
		
		--if modflag then Print("MODFLAG", 0,0,1) end
		
		--Put item in queue 
		MWTCE.push(priority,msg, modflag);
		
		-- Try it right away
		MWTCE.ResolveWho(priority == MWTCE.PRIORITY.UI);
		-- Temporary transparancy
		--MWTCE.hook.SendWho(msg);
	end

	function MWTCE.FriendsFrame_OnEvent()
		if event ~= "WHO_LIST_UPDATE" or MWTCE.ui_want_request then
			
			if event == "WHO_LIST_UPDATE" then
				-- Create a chache copy for use in the who frame
				local name, guild, level, race, class, zone;
				MWTCE.whoinfo = {};
				MWTCE.whoinfo.n, MWTCE.whoinfo.t = GetNumWhoResults();
				MWTCE.whoinfo.results = {};
				for k = 1, MWTCE.whoinfo.n, 1 do
					MWTCE.whoinfo.results[k] = { GetWhoInfo(k) };
				end
			end
		
			-- Drastic measures against hijacking mods
			MWTCE.override.FriendsFrame_OnEvent();
		else
			dbg(Print,"Relay stopped, request did not come from UI");
		end
	end

	function MWTCE.SortWho(type)
		local sortidx = {name=1, guild=2, level=3, race=4, class=5, zone=6 };
		local sfun;
		local function sortup(a,b)
			if a[sortidx[type]] < b[sortidx[type]] then
				return true;
			elseif a[sortidx[type]] > b[sortidx[type]] then
				return false;
			elseif MWTCE.lastsorttype then
				return a[sortidx[MWTCE.lastsorttype]] < b[sortidx[MWTCE.lastsorttype]]; 
			else
				return false;
			end
		end
		local function sortdown(a,b)
			if a[sortidx[type]] > b[sortidx[type]] then
				return true;
			elseif a[sortidx[type]] < b[sortidx[type]] then
				return false;
			elseif MWTCE.lastsorttype then
				return a[sortidx[MWTCE.lastsorttype]] > b[sortidx[MWTCE.lastsorttype]]; 
			else
				return false;
			end
		end

		dbg(pxd, type);
		-- Sort our cached results
		if not type or not sortidx[type] or sortidx[type] < 1 or sortidx[type] > 6 then type = "name"; end 

		if MWTCE.lastsorttype then
			if MWTCE.lastsorttype == type then
				if MWTCE.lastup then
					sfun = sortdown;
					MWTCE.lastup = false;
				else
					sfun = sortup;
					MWTCE.lastup = true;
				end
			else
				MWTCE.lastup = true;
				sfun = sortup;
			end
		else
			sfun = sortup;
			MWTCE.lastup = true;
		end
		
		table.sort(MWTCE.whoinfo.results, sfun);

		MWTCE.lastsorttype = type;
		
		MWTCE.WhoList_Update();
	end
	-------------------------------------------------------------
	-- Redefine Blizzard functions that use SendWho
	-------------------------------------------------------------
	
	-- FriendsFrame.lua 766
	function MWTCE.WhoFrameEditBox_OnEnterPressed()
		SendWho(WhoFrameEditBox:GetText(), MWTCE.PRIORITY.UI);
		WhoFrameEditBox:ClearFocus();
	end
	
	-- ChatFrame.lua 1006
	function MWTCE.SlashWho(msg)
		if ( msg == "" ) then
			msg = WhoFrame_GetDefaultWhoCommand();
			ShowWhoPanel();
		elseif ( msg == "cheat" ) then
			-- Remove the "cheat" part later!
			ShowWhoPanel();
		end
		WhoFrameEditBox:SetText(msg);
		SendWho(msg, MWTCE.PRIORITY.UI);
	end
	
	-- ItemRef.lua 2
	function MWTCE.SetItemRef(link, text, button)
		if ( strsub(link, 1, 6) == "player" ) then
			local name = strsub(link, 8);
			if ( name and (strlen(name) > 0) ) then
				name = gsub(name, "([^%s]*)%s+([^%s]*)%s+([^%s]*)", "%3");
				name = gsub(name, "([^%s]*)%s+([^%s]*)", "%2");
				if ( IsShiftKeyDown() ) then
					local staticPopup;
					staticPopup = StaticPopup_Visible("ADD_IGNORE");
					if ( staticPopup ) then 
						-- If add ignore dialog is up then enter the name into the editbox
						MWTCE.hook.SetItemRef(link, text, button);
						return;
					end
					staticPopup = StaticPopup_Visible("ADD_FRIEND");
					if ( staticPopup ) then
						-- If add ignore dialog is up then enter the name into the editbox
						MWTCE.hook.SetItemRef(link, text, button);
						return;
					end
					staticPopup = StaticPopup_Visible("ADD_GUILDMEMBER");
					if ( staticPopup ) then
						-- If add ignore dialog is up then enter the name into the editbox
						MWTCE.hook.SetItemRef(link, text, button);
						return;
					end
					staticPopup = StaticPopup_Visible("ADD_RAIDMEMBER");
					if ( staticPopup ) then
						-- If add ignore dialog is up then enter the name into the editbox
						MWTCE.hook.SetItemRef(link, text, button);
						return;
					end

					if ( not ChatFrameEditBox:IsVisible() ) then
						-- only override here
						dbg(Print, "Sending who from ItemRef");
						SendWho("n-"..name, MWTCE.PRIORITY.UI);
						return;
					end
				end
			end
		end

		MWTCE.hook.SetItemRef(link, text, button);
	end

	-- FriendsFrame.lua 253
	function MWTCE.WhoList_Update()
		-------------------------------------------------------------
		-- Locally redefined to use cache
		-------------------------------------------------------------
		local function GetWhoInfo(whoIndex)
			if MWTCE.whoinfo.results[whoIndex] then
				return unpack(MWTCE.whoinfo.results[whoIndex]);
			else
				return nil, nil, 0, nil, nil, nil, nil;
			end
		end
		local function GetNumWhoResults()
			return MWTCE.whoinfo.n, MWTCE.whoinfo.t;
		end
		-------------------------------------------------------------
		local numWhos, totalCount = GetNumWhoResults();
		local name, guild, level, race, class, zone;
		local button;
		local columnTable;
		local whoOffset = FauxScrollFrame_GetOffset(WhoListScrollFrame);
		local whoIndex;
		local showScrollBar = nil;
		if ( numWhos > WHOS_TO_DISPLAY ) then
			showScrollBar = 1;
		end
		local displayedText = "";
		if ( totalCount > MAX_WHOS_FROM_SERVER ) then
			displayedText = format(WHO_FRAME_SHOWN_TEMPLATE, MAX_WHOS_FROM_SERVER);
		end
		WhoFrameTotals:SetText(format(GetText("WHO_FRAME_TOTAL_TEMPLATE", nil, totalCount), totalCount).."  "..displayedText);
		for i=1, WHOS_TO_DISPLAY, 1 do
			whoIndex = whoOffset + i;
			button = getglobal("WhoFrameButton"..i);
			button.whoIndex = whoIndex;
			name, guild, level, race, class, zone = GetWhoInfo(whoIndex);
			columnTable = { zone, guild, race };
			getglobal("WhoFrameButton"..i.."Name"):SetText(name);
			getglobal("WhoFrameButton"..i.."Level"):SetText(level);
			getglobal("WhoFrameButton"..i.."Class"):SetText(class);
			local variableText = getglobal("WhoFrameButton"..i.."Variable");
			variableText:SetText(columnTable[UIDropDownMenu_GetSelectedID(WhoFrameDropDown)]);
			
			-- If need scrollbar resize columns
			if ( showScrollBar ) then
				variableText:SetWidth(95);
			else
				variableText:SetWidth(110);
			end

			-- Highlight the correct who
			if ( WhoFrame.selectedWho == whoIndex ) then
				button:LockHighlight();
			else
				button:UnlockHighlight();
			end
			
			if ( whoIndex > numWhos ) then
				button:Hide();
			else
				button:Show();
			end
		end

		if ( not WhoFrame.selectedWho ) then
			WhoFrameGroupInviteButton:Disable();
			WhoFrameAddFriendButton:Disable();
		else
			WhoFrameGroupInviteButton:Enable();
			WhoFrameAddFriendButton:Enable();
			WhoFrame.selectedName = GetWhoInfo(WhoFrame.selectedWho); 
		end

		-- If need scrollbar resize columns
		if ( showScrollBar ) then
			WhoFrameColumn_SetWidth(105, WhoFrameColumnHeader2);
			UIDropDownMenu_SetWidth(80, WhoFrameDropDown);
		else
			WhoFrameColumn_SetWidth(120, WhoFrameColumnHeader2);
			UIDropDownMenu_SetWidth(95, WhoFrameDropDown);
		end

		-- ScrollFrame update
		FauxScrollFrame_Update(WhoListScrollFrame, numWhos, WHOS_TO_DISPLAY, FRIENDS_FRAME_WHO_HEIGHT );

		PanelTemplates_SetTab(FriendsFrame, 2);
		ShowUIPanel(FriendsFrame);
	end

	-- FriendsFrame.lua 635
	function MWTCE.override.FriendsFrame_OnEvent()
		if ( event == "FRIENDLIST_SHOW" ) then
			FriendsList_Update();
			FriendsFrame_Update();
		elseif ( event == "FRIENDLIST_UPDATE" ) then
			FriendsList_Update();
		elseif ( event == "IGNORELIST_UPDATE" ) then
			IgnoreList_Update();
		elseif ( event == "WHO_LIST_UPDATE" ) then
			WhoList_Update();
			FriendsFrame_Update();
		elseif ( event == "GUILD_ROSTER_UPDATE" ) then
			if ( GuildFrame:IsVisible() ) then
				if ( arg1 ) then
					GuildRoster();
				end
				GuildStatus_Update();
				FriendsFrame_Update();
			end
		elseif ( event == "PLAYER_GUILD_UPDATE" ) then
			if ( FriendsFrame:IsVisible() ) then
				InGuildCheck();
			end
		elseif ( event == "GUILD_MOTD") then
			CURRENT_GUILD_MOTD = arg1;
			GuildFrameNotesText:SetText(CURRENT_GUILD_MOTD);
		end
	end

	-------------------------------------------------------------
	-- Compatibility functions, only loaded when needed
	-------------------------------------------------------------
	
	function MWTCE.compatibility.AcePlayerMenu()		
		local NeedReload = false;
	
		if AcePlayerMenu.Hooks then
			if AcePlayerMenu.Hooks.SetItemRef then
				AcePlayerMenu:Unhook("SetItemRef");
				NeedReload = true;
			end
			if AcePlayerMenu.Hooks.UnitPopup_OnClick then
				AcePlayerMenu:Unhook("UnitPopup_OnClick");
				NeedReload = true;
			end
		end
			
		function AcePlayerMenu:UnitPopup_OnClick()
			dbg(Print,"MWTCE.compatibility.AcePlayerMenu.UnitPopup_OnClick");

			local dropdownFrame = getglobal(UIDROPDOWNMENU_INIT_MENU);
			local button = this.value;
			local unit = dropdownFrame.unit;
			local name = dropdownFrame.name;

			if (button == "ADD_FRIEND") then
				AddFriend(name);
			elseif (button == "GUILD_INVITE") then
				GuildInviteByName(name);
			elseif (button == "IGNORE") then
				AddIgnore(name);
			elseif (button == "GET_NAME") then
				ChatFrameEditBox:Show();
				ChatFrameEditBox:Insert(name);
			elseif (button == "WHO") then
				SendWho(name, MWTCE.PRIORITY.UI);
			else
				return self:CallHook("UnitPopup_OnClick");
			end
			PlaySound("UChatScrollButton");
		end
		
		function AcePlayerMenu:SetItemRef(link, text, button)
			dbg(Print,"MWTCE.compatibility.AcePlayerMenu.SetItemRef");
			if ( strsub(link, 1, 6) == "player" ) then
				local name = strsub(link, 8);
				if ( name and (strlen(name) > 0) ) then
					name = gsub(name, "([^%s]*)%s+([^%s]*)%s+([^%s]*)", "%3");
					name = gsub(name, "([^%s]*)%s+([^%s]*)", "%2");
					if ( IsShiftKeyDown() ) then
						local staticPopup;
						staticPopup = StaticPopup_Visible("ADD_IGNORE");
						if ( staticPopup ) then
							-- If add ignore dialog is up then enter the name into the editbox
							getglobal(staticPopup.."EditBox"):SetText(name);
							return;
						end
						staticPopup = StaticPopup_Visible("ADD_FRIEND");
						if ( staticPopup ) then
							-- If add ignore dialog is up then enter the name into the editbox
							getglobal(staticPopup.."EditBox"):SetText(name);
							return;
						end
						staticPopup = StaticPopup_Visible("ADD_GUILDMEMBER");
						if ( staticPopup ) then
							-- If add ignore dialog is up then enter the name into the editbox
							getglobal(staticPopup.."EditBox"):SetText(name);
							return;
						end
						staticPopup = StaticPopup_Visible("ADD_RAIDMEMBER");
						if ( staticPopup ) then
							-- If add ignore dialog is up then enter the name into the editbox
							getglobal(staticPopup.."EditBox"):SetText(name);
							return;
						end
						if ( ChatFrameEditBox:IsVisible() ) then
							ChatFrameEditBox:Insert(name);
						else
							SendWho("n-"..name, MWTCE.PRIORITY.UI);					
						end
						
					else
						FriendsFrame_ShowDropdown(name, 1);
					end 
				end
				return;
			end

			return self:CallHook("SetItemRef", link, text, button);
		end
		
		if NeedReload then
			dbg(Print, "Need to reload AcePlayerMenu");
			AcePlayerMenu:Reload();
		end
	end

else
	MWCTE_CLASH = true;
end

if not MWCTE_VER_INFO then
	MWCTE_VER_INFO = {};
end
	
table.insert(MWCTE_VER_INFO, 
	{ thisloaded=not MWCTE_CLASH,  uid=MWTCE_UNIQUE_ID, embedded=MWTCE_IS_EMBEDDED, version=MWTCE_CURRENT_VERSION, mwctetype=type(MWTCE), foundvernum=tonumber(MWTCE.version), existingEQorBetter=tonumber(MWTCE.version) >= MWTCE_CURRENT_VERSION });
	